using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using System;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationCTR;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationRDTIS;


namespace Ato.CD.Inbound.CTR202302
{
    internal class CrossFormValidatorRDTIS : ICrossFormValidator
    {
        private CTR2023 ParentDocument { get; }
        private RDTIS2022 ChildDocument { get; }
        static int expenditureCap = Int32.Parse("150,000,000", System.Globalization.NumberStyles.Number); //ExpenditureCap = $150 million, Constant Tab in VR Artefacts;
        //static int refundableCap = Int32.Parse("4,000,000", System.Globalization.NumberStyles.Number);  //RefundableCap = 4000000, Constant Tab in VR Artefacts;
        static decimal refundablePremium = 0.185M; //18.5%, Constant Tab in VR Artefacts;

        internal CrossFormValidatorRDTIS(CTR2023 report, BusinessDocument childDocument)
        {          
            ParentDocument = report;
            RDTIS2022XmlConsumer consumer = new RDTIS2022XmlConsumer();
            childDocument.DocumentStream.Position = 0;
            ChildDocument = consumer.Consume(childDocument.DocumentStream, true); //childDocument.ConsumedReport;
        }
           

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();
           
            VRATOGEN402009(response);
            VRATOIEE438104(response);
            VRATOCTR500330(response);
            VRATOCTR500331(response);
            VRATOCTR500332(response);
            VRATOCTR500333(response);

            return response;
        }


        //RTDIS_CTR
        #region VR.ATO.GEN.402009
        /*
         *            IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
            RETURN VALIDATION MESSAGE
            ENDIF
      */

        private void VRATOGEN402009(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RDTIS70);
           

            if (assertion)
            {
                var processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402009",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    Location = "/tns:RDTIS/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } }
                };
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region  VR.ATO.RDTIS.438104
        /*  
             VR.ATO.RDTIS.438104
        (^RDTIS71 <> PARENT PERIOD START DATE) OR (^RDTIS72 <> PARENT PERIOD END DATE)

         */
        protected void VRATOIEE438104(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            bool assertion = ((ChildDocument.RDTIS71.HasValue && ChildDocument.RDTIS71 != ParentDocument.RPStartDate) || ChildDocument.RDTIS72.HasValue && ChildDocument.RDTIS72 != ParentDocument.RPEndDate);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.RDTIS.438104",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting period does not match the reporting period supplied on the form it was submitted with",
                    Location = "/tns:RDTIS/tns:RP/tns:PeriodStartD",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.RDTIS.438104" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS71", Value = ChildDocument.RDTIS71.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS72", Value = ChildDocument.RDTIS72.ToString() });
                response.Add(processMessage);
            }
        }

        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.CTR.500330
        /*  VR.ATO.CTR.500330 //Old rule: VR.ATO.RDTIS.438069
            Refundable R&D tax offset is not equal to 43.5% of the Refundable tax offset Total notional R&D deductions

            Technical Business Rule Format:
                IF [CTR2] = 2022 AND CTR:RP:period.startDate < ""2021-07-01"" AND [RDTIS45] <> NULL AND
                    [RDTIS45] <= 100000000 AND
                    [RDTIS46] <> ([RDTIS45] * 0.435) +/- 1
                  RETURN VALIDATION MESSAGE
                ENDIF

            Data Elements:
            ^CTR2 = CTR:RP:pyin.02.37:Report.TargetFinancial.Year
            ^RDTIS46 = RDTIS:RP:RefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
            ^RDTIS45 = RDTIS:RP:RefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalRefundable.Amount
        */
        public virtual void VRATOCTR500330(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            int dateCompareResult = DateTime.Compare(ParentDocument.RPStartDate.GetValueOrDefault(), DateTime.Parse("2021-07-01"));
            Decimal refundableOffset = 0;
            bool assertion = false;
            if (ParentDocument.CTR2 != null && ParentDocument.CTR2.GetValueOrDefault() == 2022 &&
                ParentDocument.RPStartDate != null && dateCompareResult < 0 &&
                ChildDocument.RDTIS45 != null && ChildDocument.RDTIS45.GetValueOrDefault() <= 100000000)
            {
                refundableOffset = ChildDocument.RDTIS45.GetValueOrDefault() * 0.435M;
                assertion = OutsideRange(ChildDocument.RDTIS46.GetValueOrDefault(), refundableOffset, 1);
            }

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500330",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset for 2022 Early Balancer is incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500330" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR2", Value = ParentDocument.CTR2.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR:RP:period.startDate", Value = ParentDocument.RPStartDate.GetValueOrDefault().ToString("yyyy-MM-dd") });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS45", Value = GetValueOrEmpty(ChildDocument.RDTIS45) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS46", Value = GetValueOrEmpty(ChildDocument.RDTIS46) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "Calculated Refundable R&D Tax Offset", Value = refundableOffset.ToString() });
                response.Add(processMessage);
            }
        }
        #endregion //VR.ATO.CTR.500330


        #region VR.ATO.CTR.500331
        /*  VR.ATO.CTR.500331 //Old rule: VR.ATO.RDTIS.438071
            Non-refundable R&D tax offset is not equal to 38.5% of the Non-refundable tax offset Total notional R&D deductions

            Technical Business Rule Format:
            IF [CTR2] = 2022 AND CTR:RP:period.startDate < "2021-07-01" AND  [RDTIS47] <> NULL AND
                [RDTIS47] <= 100000000 AND
                [RDTIS48] <> ([RDTIS47] * 0.385) +/-1
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:
            ^CTR2 = CTR:RP:pyin.02.37:Report.TargetFinancial.Year
            ^RDTIS47 = RDTIS:RP:NonRefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetNotionalNonRefundable.Amount
            ^RDTIS48 = RDTIS:RP:NonRefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
        */
        public virtual void VRATOCTR500331(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            int dateCompareResult = DateTime.Compare(ParentDocument.RPStartDate.GetValueOrDefault(), DateTime.Parse("2021-07-01"));
            decimal NonRefundableOffset = 0;
            bool assertion = false;

            if (ParentDocument.CTR2 != null && ParentDocument.CTR2.GetValueOrDefault() == 2022 &&
                ParentDocument.RPStartDate != null && dateCompareResult < 0 &&
                ChildDocument.RDTIS47 != null && ChildDocument.RDTIS47.GetValueOrDefault() <= 100000000)
            {
                NonRefundableOffset = ChildDocument.RDTIS47.GetValueOrDefault() * 0.385M;
                assertion = OutsideRange(ChildDocument.RDTIS48.GetValueOrDefault(), NonRefundableOffset, 1);
            }

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500331",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset for 2022 Early Balancer is incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500331" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR2", Value = ParentDocument.CTR2.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR:RP:period.startDate", Value = ParentDocument.RPStartDate.GetValueOrDefault().ToString("yyyy-MM-dd") });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS47", Value = GetValueOrEmpty(ChildDocument.RDTIS47) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS48", Value = GetValueOrEmpty(ChildDocument.RDTIS48) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "Calculated Non-refundable R&D Tax Offset", Value = NonRefundableOffset.ToString() });
                response.Add(processMessage);
            }
        }
        #endregion //VR.ATO.CTR.500331

        #region VR.ATO.CTR.500332 
        /*  VR.ATO.CTR.500332 //Old rule: VR.ATO.RDTIS.438114
        
        Refundable R&D tax offset calculated value is incorrect

        Technical Business Rule Format:
            IF (([CTR2] = SET(2022,2023) AND CTR:RP:period.startDate >= "2021-07-01")) AND 
                [RDTIS36] = FALSE AND [RDTIS37] = FALSE AND [RDTIS46] > 0 AND 
                [RDTIS46] <> (RefundableOffsetCalc() +/- 1)
                RETURN VALIDATION MESSAGE
            ENDIF
        
        Data Elements:
            ^CTR2 = CTR:RP:pyin.02.37:Report.TargetFinancial.Year
            ^RDTIS36 = RDTIS:RP:AggregatedTurnover:TaxConcession.ExemptEntityOwnership.Indicator
            ^RDTIS37 = RDTIS:RP:AggregatedTurnover:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
            ^RDTIS46 = RDTIS:RP:RefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetRefundable.Amount
        */
        public void VRATOCTR500332(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            int dateCompareResult = DateTime.Compare(ParentDocument.RPStartDate.GetValueOrDefault(), DateTime.Parse("2021-07-01"));
            Decimal refundableOffset = 0;
            bool assertion = false;
            if ((ParentDocument.CTR2 != null && ParentDocument.CTR2.GetValueOrDefault() == 2022 || ParentDocument.CTR2.GetValueOrDefault() == 2023) &&
                 ParentDocument.RPStartDate != null && dateCompareResult >= 0 &&
                 ChildDocument.RDTIS36 != null && ChildDocument.RDTIS36 == false &&
                 ChildDocument.RDTIS37 != null && ChildDocument.RDTIS37 == false &&
                 ChildDocument.RDTIS46 != null && ChildDocument.RDTIS46.GetValueOrDefault() > 0)
            {
                refundableOffset = CalcRefundableOffset(ChildDocument, true);
                assertion = OutsideRange(ChildDocument.RDTIS46.GetValueOrDefault(), refundableOffset, 1); // Get RefundableOffset
            }
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500332",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Refundable R&D tax offset calculated value is incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500332" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR2", Value = ParentDocument.CTR2.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR:RP:period.startDate", Value = ParentDocument.RPStartDate.GetValueOrDefault().ToString("yyyy-MM-dd") });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS36", Value = ChildDocument.RDTIS36.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS37", Value = ChildDocument.RDTIS37.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS46", Value = GetValueOrEmpty(ChildDocument.RDTIS46) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "Calculated Refundable R&D Tax Offset", Value = refundableOffset.ToString() });
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.500332

        #region VR.ATO.CTR.500333
        /*  VR.ATO.CTR.500333  //VR.ATO.CTR.438115
            Non-refundable R&D tax offset calculated value is incorrect

            Technical Business Rule Format:
                IF (([CTR2] = SET(2022,2023) AND CTR:RP:period.startDate >= "2021-07-01")) AND 
                    ([RDTIS36] = TRUE OR [RDTIS37] = TRUE) AND [RDTIS48] > 0 AND 
                     [RDTIS48] <> (NonRefundableOffsetCalc() +/- 1)
                    RETURN VALIDATION MESSAGE
                ENDIF
            
            Data Elements:
                ^CTR2 = CTR:RP:pyin.02.37:Report.TargetFinancial.Year
                ^RDTIS36 = RDTIS:RP:AggregatedTurnover:TaxConcession.ExemptEntityOwnership.Indicator
                ^RDTIS37 = RDTIS:RP:AggregatedTurnover:TaxConcession.AggregatedTurnoverAtOrAboveThreshold.Indicator
                ^RDTIS48 = RDTIS:RP:NonRefundableTaxOffset:TaxConcession.ResearchAndDevelopment.TaxOffsetNonRefundable.Amount
         */
        public void VRATOCTR500333(List<ProcessMessageDocument> response)
        {
            ProcessMessageDocument processMessage;
            int dateCompareResult = DateTime.Compare(ParentDocument.RPStartDate.GetValueOrDefault(), DateTime.Parse("2021-07-01"));
            decimal NonRefundableOffset = 0;
            bool assertion = false;
            if ((ParentDocument.CTR2 != null && ParentDocument.CTR2.GetValueOrDefault() == 2022 || ParentDocument.CTR2.GetValueOrDefault() == 2023) &&
                 (ParentDocument.RPStartDate != null && dateCompareResult >= 0) &&
                 (ChildDocument.RDTIS36 != null && ChildDocument.RDTIS36 == true ||
                  ChildDocument.RDTIS37 != null && ChildDocument.RDTIS37 == true) &&
                 (ChildDocument.RDTIS48 != null && ChildDocument.RDTIS48.GetValueOrDefault() > 0))
            {
                NonRefundableOffset = calcNonRefundableOffset(ChildDocument);
                assertion = OutsideRange(ChildDocument.RDTIS48.GetValueOrDefault(), NonRefundableOffset, 1);
            }
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.CTR.500333",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-refundable R&D tax offset calculated value is incorrect",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CTR.500333" } },
                };

                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR2", Value = ParentDocument.CTR2.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "CTR:RP:period.startDate", Value = ParentDocument.RPStartDate.GetValueOrDefault().ToString("yyyy-MM-dd") });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS36", Value = ChildDocument.RDTIS36.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS37", Value = ChildDocument.RDTIS37.ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "RDTIS48", Value = GetValueOrEmpty(ChildDocument.RDTIS48) });
                processMessage.Parameters.Add(new ProcessMessageParameter { Name = "Calculated Non-refundable R&D Tax Offset", Value = NonRefundableOffset.ToString() });
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.CTR.500333

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }

        #region CalcRefundableOffset

        /*
        IF ([RDTIS45] < ExpenditureCap) THEN
             [RDTIS45] * ([RDTIS78] + RefundablePremium)
        ELSE 
             (ExpenditureCap * ([RDTIS78] + RefundablePremium)) + (([RDTIS45] - ExpenditureCap) * [RDTIS78])
        ENDIF
        */
        private decimal CalcRefundableOffset(RDTIS2022 report, bool isRefundableOffset)
        {
            if (ChildDocument.RDTIS45 < expenditureCap)
            {
                return (decimal)(ChildDocument.RDTIS45.GetValueOrDefault() * ((ChildDocument.RDTIS78.GetValueOrDefault() / 100) + refundablePremium));
            }
            else
            {
                return (decimal)((expenditureCap * ((ChildDocument.RDTIS78.GetValueOrDefault() / 100) + refundablePremium)) + ((ChildDocument.RDTIS45.GetValueOrDefault() - expenditureCap) * (ChildDocument.RDTIS78.GetValueOrDefault() / 100)));
            }
        }

        #endregion

        #region calcNonRefundableOffset

        /*
          IF ([RDTIS47] > ExpenditureCap) THEN
                 Intensity = (ExpenditureCap / [RDTIS81])
                 ExcessOffset = ([RDTIS47] - ExpenditureCap) * [RDTIS78]
            ELSE
                 Intensity = ([RDTIS47] / [RDTIS81])
                 ExcessOffset = 0
            ENDIF            
           
            IF (Intensity > 0 AND Intensity <= T1Intensity) THEN
                 IF ([RDTIS47] > ExpenditureCap) THEN
                      T1Deductions = ExpenditureCap
                 ELSE
                      T1Deductions = [RDTIS47]
                 ENDIF
            ELSE
                 T1Deductions = T1Intensity * [RDTIS81]
                      IF [RDTIS47] > ExpenditureCap THEN
                           T2Deductions = (ExpenditureCap - T1Deductions)
                      ELSE
                           T2Deductions = ([RDTIS47] - T1Deductions)
                      ENDIF
            ENDIF

            T1Offset = (T1Deductions * ([RDTIS78] + T1Premium))
            T2Offset = (T2Deductions * ([RDTIS78] + T2Premium))

            NonRefundableOffset = T1Offset + T2Offset + ExcessOffset

         */
        private decimal calcNonRefundableOffset(RDTIS2022 report)
        {
            decimal intensity = 0, excessOffset = 0;
            // T1,T2  Deduction Calculation Variable
            decimal t1Intensity = 0.02M; // 2%;
            //decimal t2Intensity = 0.02+M; // 2+% ;  
            decimal t1Deductions = 0, t2Deductions = 0;
            // Premium %
            decimal t1Premium = 0.085M; // 8.5%;
            decimal t2Premium = 0.165M; // 16.5%;

            //decimal taxRate = (ChildDocument.RDTIS78.GetValueOrDefault() / 100); //RDTIS78

            // intensity and excessOffset Calculation
            if (ChildDocument.RDTIS47.GetValueOrDefault() > expenditureCap)
            {
                if (ChildDocument.RDTIS81.GetValueOrDefault() > 0)
                    intensity = (expenditureCap / ChildDocument.RDTIS81.GetValueOrDefault());
                excessOffset = (ChildDocument.RDTIS47.GetValueOrDefault() - expenditureCap) * (ChildDocument.RDTIS78.GetValueOrDefault() / 100);
            }
            else
            {
                if (ChildDocument.RDTIS81.GetValueOrDefault() > 0)
                    intensity = (ChildDocument.RDTIS47.GetValueOrDefault() / ChildDocument.RDTIS81.GetValueOrDefault());
                excessOffset = 0;
            }

            // T1,T2 Deduction Calculation             
            if (intensity > 0 && intensity <= t1Intensity) // Tier1 deduction
            {
                t1Deductions = ChildDocument.RDTIS47.GetValueOrDefault() > expenditureCap ? expenditureCap : ChildDocument.RDTIS47.GetValueOrDefault();
            }
            else
            {
                t1Deductions = t1Intensity * ChildDocument.RDTIS81.GetValueOrDefault();
                //IF[RDTIS47] > ExpenditureCap THEN
                //         T2Deductions = (ExpenditureCap - T1Deductions)
                //      ELSE
                //           T2Deductions = ([RDTIS47] - T1Deductions)
                //      ENDIF
                //if (intensity > t1Intensity )//&& intensity <= t2Intensity) // Tier2 deduction
                //{
                t2Deductions = ChildDocument.RDTIS47.GetValueOrDefault() > expenditureCap ? (expenditureCap - t1Deductions) : (ChildDocument.RDTIS47.GetValueOrDefault() - t1Deductions);
                //}
                //else
                //{
                //    t2Deductions = (t2Intensity * ChildDocument.RDTIS81.GetValueOrDefault()) - t1Deductions;

                //}
            }
            // t1Offset + t2Offset + excessOffset  
            return Math.Round(((t1Deductions * ((ChildDocument.RDTIS78.GetValueOrDefault() / 100) + t1Premium)) + (t2Deductions * ((ChildDocument.RDTIS78.GetValueOrDefault() / 100) + t2Premium)) + excessOffset), 2, MidpointRounding.AwayFromZero);
        }

        #endregion
    }
}